/*
 * This system procedure is used to return the list of columns
 * and the corresponding column for the given table(s).
 */
 
CREATE PROCEDURE psp_columns (
	in :database_qual   VARCHAR(20) = DATABASE(), -- Name of the database
	in :table_name      VARCHAR(255), -- Name of the table
	in :column_name     VARCHAR(255)=NULL -- Name of the column
)
RETURNS (
	TABLE_QUALIFIER     VARCHAR (20), 	-- Name of the database that contains the table
	TABLE_OWNER         VARCHAR (20), 	-- Owner of the table
	TABLE_NAME          VARCHAR (255),	-- Table Name
	COLUMN_NAME         VARCHAR(255),	-- Column name
	DATA_TYPE           SMALLINT,		-- Datatype of Column
	TYPE_NAME           VARCHAR (32),		-- Datatype Name
	PRECISION           INTEGER,		-- Precision
	LENGTH              INTEGER, 		-- Length
	SCALE               SMALLINT,			-- Scale
	RADIX               SMALLINT,			-- Radix
	NULLABLE            SMALLINT,		-- Nullable flag
	REMARKS             VARCHAR(255)		-- Remarks
)
as 
BEGIN 
    -- Variable to hold the dynamic query
    DECLARE :stmt_psp_columns LONGVARCHAR;

    -- Check if the database qualifier specified is a blank string
	if (rtrim(:database_qual) = '') then
		SIGNAL 'S1000', 'Please enter a valid database name. Database name cannot be a blank string'; 
	end if;
    
     -- Check if the table_name is null
    if (:table_name is null) then
    	SIGNAL 'S1000', 'Table name cannot be null'; 
    end if;
    
     -- Check if the table_name specified is a blank string
	if (rtrim(:table_name) = '') then
		SIGNAL 'S1000', 'Table name cannot be a blank string'; 
	end if;

    -- Check if the table_name specified is a blank string
	if (rtrim(:column_name) = '') then
		SIGNAL 'S1000', 'Column name cannot be a blank string'; 
	end if;

    SET :database_qual = IFNULL(:database_qual, DATABASE());

    SET :column_name = IFNULL(:column_name,'%'); 

    SET :stmt_psp_columns = 'select ''' + :database_qual + 
				''' ,null,' +
				' rtrim(xf$name),'+
				' rtrim(xe$name), '+
				' case xe$datatype ' +
					' when 0 then if(xe$flags & 4096 = 4096, -2, 1) '+
					' when 1 then '+
						' case xe$size '+
						' when 1 then -6 '+
						' when 2 then 5  '+
						' when 4 then 4 '+
						' else -5 '+
						' end '+
					' when 2 then'+
						' case xe$size '+
						' when 4 then 7 '+
						' else 8  '+
						' end '+
					' when 3 then 9 '+
					' when 4 then 10 '+
					' when 5 then 3 '+
					' when 6 then 3 '+
					' when 7 then -7 '+
					' when 8 then 2 '+
					' when 9 then ' + 
						' case xe$size ' +
						' when 4 then 7 ' +
						' else 8 ' +
						' end' +
					' when 10 then 12 '+
					' when 11 then 12 '+
					' when 12 then -1 '+
					' when 13 then -1 '+
					' when 14 then'+
						' case xe$size '+
						' when 1 then -6 '+
						' when 2 then 5  '+
						' when 4 then 4 '+
						' else -5 '+
						' end '+
					' when 15 then ' + 
						' case xe$size ' +
						' when 2 then 5 ' +
						' else 4 ' +
						' end' +
					' when 16 then -7 '+
					' when 17 then 2 ' +
					' when 18 then 2 ' +
					' when 19 then 3 '+
					' when 20 then 11 '+
					' when 21 then if(xe$flags & 4096 = 4096, -4, -1) '+
					' when 25 then -3 '  +
					' when 26 then -3 ' + 
					' when 27 then -11 ' + 
					' when 28 then 2 ' +
					' when 29 then 2 ' +
					' when 30 then 11 '+
					' when 31 then 2 ' +
				' end as SQL_DATA_TYPE,'+
				' case xe$datatype '+
					' when 0 then if(xe$flags & 4096 = 4096, ' + '''BINARY''' +
                        ', ' + '''CHAR''' + ')' +
					' when 1 then'+
						' case xe$size '+
						' when 1 then ' + '''TINYINT''' +
						' when 2 then ' + '''SMALLINT''' +
						' when 4 then '+ '''INTEGER''' +
						' else ' + '''BIGINT''' +
						' end '+
					' when 2 then'+
						' case xe$size '+
						' when 4 then ' + '''REAL''' +
						' else ' + '''DOUBLE''' +
						' end'+
					' when 3 then ' + '''DATE''' +
					' when 4 then ' + '''TIME''' +
					' when 5 then ' + '''DECIMAL''' +
					' when 6 then ' + '''MONEY''' +
					' when 7 then ' + '''BIT''' +
					' when 8 then ' + '''NUMERIC''' +
					' when 9 then ' + 
						' case xe$size ' +
						' when 4 then' + '''BFLOAT4''' +
						' when 8 then' + '''BFLOAT8''' +
						'end' +
					' when 10 then ' + '''VARCHAR''' +
					' when 11 then ' + '''VARCHAR''' +
					' when 12 then ' + '''LONGVARCHAR''' +
					' when 13 then ' + '''LONGVARCHAR''' +
					' when 14 then'+
						' case xe$size '+
						' when 1 then ' + '''UTINYINT''' +
						' when 2 then ' + '''USMALLINT''' +
						' when 4 then ' + '''UINTEGER''' +
						' else ' + '''UBIGINT''' +
						' end '+
					' when 15 then'+
						' case xe$size '+
						' when 2 then ' + '''SMALLIDENTITY''' +
						' else ' + '''IDENTITY''' +
						' end '+
					' when 16 then ' + '''BIT''' +
					' when 17 then ' + '''NUMERICSTS''' +
					' when 18 then ' + '''NUMERICSA''' +
					' when 19 then ' + '''CURRENCY''' +
					' when 20 then ' + '''TIMESTAMP''' +
					' when 21 then if(xe$flags & 4096 = 4096, ' + 
                    '''LONGVARBINARY''' + ' , ' + '''LONGVARCHAR''' + ')' +
					' when 25 then ' + '''BCHAR''' +
					' when 26 then ' + '''BVARCHAR''' +
					' when 27 then ' + '''UNIQUEIDENTIFIER''' +
					' when 28 then ' + '''NUMERICSLB''' +
					' when 29 then ' + '''NUMERICSLS''' +
					' when 30 then ' + '''DATETIME''' +
					' when 31 then ' + '''NUMERICSTB''' +
				' end,'+
				' case xe$datatype '+
					' when 0 then xe$size'+
					' when 1 then'+
						' case xe$size '+
						' when 1 then 3 '+
						' when 2 then 5 '+
						' when 4 then 10 '+
						' else 19 '+
						' end '+
					' when 2 then'+
						' case xe$size '+
						' when 4 then 7 '+
						' else 15 '+
						' end'+
					' when 3 then 10 '+
					' when 4 then 8 '+
					' when 5 then'+
						' case when xe$flags & 8192 = 8192 '+
						' then (xe$size * 2) - 2 '+
						' else (xe$size * 2) - 1 '+
						' end'+
					' when 6 then'+
						' case when xe$flags & 8192 = 8192 '+
						' then (xe$size * 2) - 2 '+
						' else (xe$size * 2) - 1 '+
						' end'+
					' when 7 then 1 '+
					' when 8 then xe$size '+
					' when 9 then'+
						' case xe$size '+
						' when 4 then 7 '+
						' else 15 '+
						' end'+
					' when 10 then (xe$size - 1) '+
					' when 11 then (xe$size - 1) '+
					' when 12 then (xe$size - 1) '+
					' when 13 then (xe$size - 4) '+
					' when 14 then'+
						' case xe$size '+
						' when 1 then 3 '+
						' when 2 then 5 '+
						' when 4 then 10 '+
						' else 20 '+
						' end '+
					' when 15 then'+
						' case xe$size '+
						' when 2 then 5 '+
						' else 10 '+
						' end '+
					' when 16 then 1 '+
					' when 17 then xe$size - 1 '+
					' when 18 then xe$size '+
					' when 19 then 19 '+
					' when 20 then 19 '+
					' when 21 then 2147483647 '+
					' when 27 then 36 '+
					' when 28 then xe$size '+
					' when 29 then xe$size - 1 '+
					' when 30 then 19 '+
					' when 31 then xe$size '+
					' else null'+
				' end,'+
				' case xe$datatype '+
					' when 0 then xe$size'+
					' when 1 then'+
						' case xe$size '+
						' when 1 then 1 '+
						' when 2 then 2 '+
						' when 4 then 4 '+
						' else 8 '+
						' end '+
					' when 2 then xe$size'+
					' when 3 then 6 '+
					' when 4 then 6  '+
					' when 5 then'+
						' case when xe$flags & 8192 = 8192 '+
						' then (xe$size * 2) '+
						' else (xe$size * 2) + 1 '+
						' end'+
					' when 6 then'+
						' case when xe$flags & 8192 = 8192 '+
						' then (xe$size * 2) '+
						' else (xe$size * 2) + 1 '+
						' end'+
					' when 7 then 1 '+
					' when 8 then xe$size + 2 '+
					' when 9 then xe$size'+
					' when 10 then (xe$size - 1) '+
					' when 11 then (xe$size - 1) '+
					' when 12 then (xe$size - 1) '+
					' when 13 then (xe$size - 4) '+
					' when 14 then'+
						' case xe$size '+
						' when 1 then 1 '+
						' when 2 then 2 '+
						' when 4 then 4 '+
						' else 8 '+
						' end '+
					' when 15 then'+
						' case xe$size '+
						' when 2 then 2 '+
						' else 4 '+
						' end '+
					' when 16 then 1 '+
					' when 17 then xe$size + 1 '+
					' when 18 then xe$size + 2 '+
					' when 19 then 21 '+
					' when 20 then 16 '+
					' when 21 then 2147483647 '+
					' when 25 then 2147483647 '+
					' when 26 then 2147483647 '+
					' when 27 then 16 '+
					' when 28 then xe$size + 2 '+
					' when 29 then xe$size + 1 '+
					' when 30 then 16 '+
					' when 31 then xe$size + 2 '+
					' else null'+
				' end,'+
				' case xe$datatype '+
					' when 1 then 0 '+
					' when 3 then null  '+
					' when 4 then null  '+
					' when 5 then xe$dec'+
					' when 6 then xe$dec'+
					' when 7 then 0 '+
					' when 8 then xe$dec '+
					' when 14 then 0 '+
					' when 15 then 0 '+
					' when 16 then 0 '+
					' when 17 then xe$dec '+
					' when 18 then xe$dec '+
					' when 19 then 4 '+
					' when 20 then 0 '+
					' when 27 then 0 '+
					' when 28 then xe$dec '+
					' when 29 then xe$dec '+
					' when 30 then 0 '+
					' when 31 then xe$dec '+
					' else null'+
				' end,	'+
				' case xe$datatype '+
					' when 0 then null '+
					' when 3 then null '+
					' when 4 then null '+
					' when 7 then 2	 '+
					' when 10 then null '+
					' when 11 then null '+
					' when 12 then null '+
					' when 13 then null '+
					' when 16 then 2	 '+
					' when 20 then null '+
					' when 21 then null '+
					' when 27 then null '+
					' when 30 then null '+
					' else 10 '+
				' end,'+
				' if (xe$flags & 4 = 4, 1, 0),'+
				' null'+
				' FROM "' + :database_qual + '".X$File, "' + :database_qual +
				'".X$Field WHERE Xf$Name LIKE ''' +  :table_name + 
				''' AND Xf$Id = Xe$File AND Xe$Name LIKE ''' + :column_name + 
				    ''' AND Xe$DataType < 227 ORDER BY Xf$Name, Xe$Offset;'

    exec (:stmt_psp_columns);

END; -- End of stored procedure.

